# https://wiki.selfhtml.org/wiki/MIME-Type/%C3%9Cbersicht
SUFFIX_MEDIA_TYPE_MAPPING = {
    ".html": "text/html",
    ".css": "text/css",
    ".js": "text/javascript",
    ".txt": "text/plain",
    # A
    ".dwg": "application/acad",
    ".asd": "application/astound",
    ".asn": "application/astound",
    ".tsp": "application/dsptype",
    # image
    ".png": "image/png",
    ".gif": "image/gif",
    ".ief": "image/ief",
    ".jpeg": "image/jpeg",
    ".jpg": "image/jpeg",
    ".jpe": "image/jpeg",
    ".svg": "image/svg+xml",
    ".tif": "image/tif",
    ".tiff": "image/tif",
    ".vasa": "image/mcf",
    ".bmp": "image/bmp",
    ".cod": "image/cis-cod",
    ".ras": "image/cmu-raster",
    ".fif": "image/fif",
    ".wbmp": "image/vnd.wap.wbmp",
    ".fh4": "image/x-freehand",
    ".fh5": "image/x-freehand",
    ".fhc": "image/x-freehand",
    ".ico": "image/x-icon",
    ".pnm": "image/x-portable-anymap",
    ".pbm": "image/x-portable-bitmap",
    ".pgm": "image/x-portable-graymap",
    ".ppm": "image/x-portable-pixmap",
    ".rgb": "image/x-rgb",
    ".xwd": "image/x-windowdump",
    ".xbm": "image/x-xbitmap",
    ".xpm": "image/x-xpixmap",
}
